(self.webpackChunkPaddleLabel_Frontend=self.webpackChunkPaddleLabel_Frontend||[]).push([[124],{91220:function(i,s,t){"use strict";t.d(s,{Z:function(){return u}});var O=t(64254);function u(_,E){var e;if(typeof Symbol=="undefined"||_[Symbol.iterator]==null){if(Array.isArray(_)||(e=(0,O.Z)(_))||E&&_&&typeof _.length=="number"){e&&(_=e);var l=0,a=function(){};return{s:a,n:function(){return l>=_.length?{done:!0}:{done:!1,value:_[l++]}},e:function(P){throw P},f:a}}throw new TypeError(`Invalid attempt to iterate non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}var o=!0,d=!1,n;return{s:function(){e=_[Symbol.iterator]()},n:function(){var P=e.next();return o=P.done,P},e:function(P){d=!0,n=P},f:function(){try{!o&&e.return!=null&&e.return()}finally{if(d)throw n}}}}},41435:function(i,s,t){"use strict";t.d(s,{Z:function(){return l}});var O=t(94663),u=t(80112);function _(a){return Function.toString.call(a).indexOf("[native code]")!==-1}var E=t(18597);function e(a,o,d){return(0,E.Z)()?e=Reflect.construct:e=function(r,P,c){var M=[null];M.push.apply(M,P);var m=Function.bind.apply(r,M),f=new m;return c&&(0,u.Z)(f,c.prototype),f},e.apply(null,arguments)}function l(a){var o=typeof Map=="function"?new Map:void 0;return l=function(n){if(n===null||!_(n))return n;if(typeof n!="function")throw new TypeError("Super expression must either be null or a function");if(typeof o!="undefined"){if(o.has(n))return o.get(n);o.set(n,r)}function r(){return e(n,arguments,(0,O.Z)(this).constructor)}return r.prototype=Object.create(n.prototype,{constructor:{value:r,enumerable:!1,writable:!0,configurable:!0}}),(0,u.Z)(r,n)},l(a)}},41180:function(i){i.exports={ppcard:"ppcard___27hGd",title:"title___22R8f"}},70362:function(i){i.exports={card:"card___CFZWU",thumbnail:"thumbnail___1gIPM",button:"button___g0lkO"}},48627:function(i){i.exports={container:"container___2RXc3"}},17969:function(i){i.exports={col:"col___yKN-b"}},31982:function(i,s,t){"use strict";var O=t(89032),u=t(15746),_=t(11849),E=t(13062),e=t(71230),l=t(11700),a=t(67294),o=t(41180),d=t.n(o),n=t(85893),r=function(c){return(0,n.jsxs)("div",{className:d().ppcard,style:c.style,hidden:c.hidden,children:[(0,n.jsx)(e.Z,{className:d().titleRow,style:{display:c.title?void 0:"none"},children:(0,n.jsx)(l.Z,{className:d().title,children:c.title})}),(0,n.jsx)(e.Z,{style:{marginTop:26},children:(0,n.jsx)(u.Z,{span:24,style:(0,_.Z)({paddingLeft:30,paddingRight:30,textAlign:"center"},c.innerStyle),children:c.children})})]})};s.Z=r},40318:function(i,s,t){"use strict";var O=t(57663),u=t(71577),_=t(48971),E=t(67294),e=t(70362),l=t.n(e),a=t(85893),o=function(n){return(0,a.jsxs)("div",{className:l().card,style:{height:n.height,width:n.width},onClick:n.onClick?n.onClick:function(){return _.m8.push(n.href?n.href:"")},children:[(0,a.jsx)("img",{className:l().thumbnail,alt:n.wording||l().thumbnail,src:n.imgSrc,style:{height:n.height,width:n.width}}),(0,a.jsx)(u.Z,{className:l().button,style:{width:n.width},children:n.children})]})};s.Z=o},11428:function(i,s,t){"use strict";var O=t(67294),u=t(48627),_=t.n(u),E=t(85893),e=function(a){return(0,E.jsx)("div",{className:"".concat(_().container),style:{backgroundImage:"url(./pics/background.png)"},children:a.children})};s.Z=e},36505:function(i,s,t){"use strict";t.d(s,{I:function(){return _}});var O=t(48971),u=t(85893),_=function(e){var l=(0,O.YB)();return function(a){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:e,d=a==null||a==""?o:o+"."+a;return(0,u.jsx)("p",{"data-test-id":d,style:{display:"inline"},children:l.formatMessage({id:d})})}}},52940:function(i,s,t){"use strict";var O=t(11849),u=t(89032),_=t(15746),E=t(2824),e=t(67294),l=t(17969),a=t.n(l),o=t(85893),d=function(r){var P=(0,e.useState)(!1),c=(0,E.Z)(P,2),M=c[0],m=c[1];return(0,o.jsx)(_.Z,(0,O.Z)((0,O.Z)({},r),{},{className:"".concat(a().col," ").concat(r.className),style:{zIndex:M?11:10,width:"100%"},onMouseOver:function(){m(!0)},onMouseLeave:function(){m(!1)},children:r.children}))};s.Z=d},64873:function(i,s,t){"use strict";t.r(s);var O=t(89032),u=t(15746),_=t(13062),E=t(71230),e=t(34792),l=t(48086),a=t(67294),o=t(48971),d=t(40318),n=t(31982),r=t(11428),P=t(52940),c=t(15156),M=t(36505),m=t(85893),f=function(){var v=(0,M.I)("pages.welcome");return(0,m.jsx)(r.Z,{children:(0,m.jsx)(E.Z,{style:{marginTop:20},children:(0,m.jsx)(u.Z,{span:24,children:(0,m.jsx)(n.Z,{style:{height:500},title:v("sampleProject"),children:(0,m.jsx)(E.Z,{children:Object.entries(c.ux).map(function(h){var D=h[0],I=h[1];return(0,m.jsx)(P.Z,{span:4,children:(0,m.jsx)(d.Z,{height:360,width:310,imgSrc:I.avatar,onClick:D!="keypointDetection"?function(){c.bY.loadSample({taskCategoryId:I.id}).then(function(L){o.m8.push("/project_overview?projectId=".concat(L.projectId))})}:function(){l.default.info(v("underDevelopment","global"))},children:v(D,"global")})},D)})})})})})})};s.default=f}}]);
