.container___2RXc3{width:100%;padding:1.25rem 2.5rem 1.875rem;background-repeat:no-repeat;background-size:cover}.ant-pro-basicLayout-content{display:flex;margin:0}.card___CFZWU{width:12.5rem;border:none;cursor:pointer}.card___CFZWU .ant-card-body{margin:.875rem 0 0;padding:.438rem;font-weight:400;font-size:1.1rem;text-align:center;border:.125rem solid #f0f0f0;border-radius:.125rem}.card___CFZWU .thumbnail___1gIPM{width:100%;height:15rem;box-shadow:0 .125rem .375rem 0 rgba(0,0,0,.27)}.card___CFZWU .button___g0lkO{width:12.5rem;height:2.5rem;margin-top:.875rem;padding:0;font-size:.85rem;border-radius:.125rem;box-shadow:0 .125rem .375rem 0 rgba(0,0,0,.27)}.ppcard___27hGd{padding:1.25rem 0;background-color:#fff;box-shadow:0 .125rem .063rem 0 rgba(0,0,0,.26)}.ppcard___27hGd .title___22R8f{height:2.5rem;margin-bottom:0;padding-left:1.063rem;font-weight:400;font-size:1.25rem;line-height:2.5rem;border-left:.375rem solid #1727c2}.table___BHQO2 tr:nth-child(2n){background:#e8eaed}.table___BHQO2 tr:nth-child(odd){background:#f9f9f9}.table___BHQO2 .ant-table-tbody>tr.ant-table-row:hover>td,.table___BHQO2 .ant-table-tbody>tr>td.ant-table-cell-row-hover{background:none}.table___BHQO2 .ant-table-tbody>tr>td,.table___BHQO2 .ant-table-thead>tr>th,.table___BHQO2 .ant-table tfoot>tr>td,.table___BHQO2 .ant-table tfoot>tr>th{padding:1rem 1.875rem}.table___BHQO2 .ant-select-single:not(.ant-select-customize-input) .ant-select-selector{padding:0}.table___BHQO2 .ant-select:not(.ant-select-customize-input) .ant-select-selector{border-radius:.25rem}.table___BHQO2 .ant-select-arrow{right:.5rem;color:#000}.table___BHQO2 .ant-table-placeholder{display:none}.pagination___1KJhU{margin-left:1.25rem}.pagination___1KJhU .pageSizeSelector___2XZ11{width:4.125rem;height:2.25rem;margin-top:.25rem;margin-right:.313rem;margin-left:.313rem}.pagination___1KJhU .pageSizeSelector___2XZ11 .ant-select-selector{height:2.25rem}.pagination___1KJhU .pageSizeSelector___2XZ11 .ant-select-selector .ant-select-selection-item,.pagination___1KJhU .pageSizeSelector___2XZ11 .ant-select-selector .ant-select-selection-placeholder{line-height:2rem}.pagination___1KJhU .ant-btn{width:4.125rem;height:2.25rem;padding:0;font-size:.875rem;border:.063rem solid #d8d8d8;border-radius:.25rem}.pagination___1KJhU .ant-pagination-item{width:2.5rem;height:2.25rem;margin-top:.25rem;border:.063rem solid #d8d8d8;border-radius:.25rem}.pagination___1KJhU .ant-pagination-item a{line-height:2.063rem}.button___3gM4r{padding:0 .375rem;font-size:.875rem;border-radius:.25rem}#createBtn___UiGiR{min-width:13.75rem;min-height:3.75rem;padding:0;color:#1727c2;font-size:1.625rem;line-height:3.75rem;border-radius:.125rem}#createBtn___UiGiR .anticon{font-size:1.125rem;line-height:3.75rem;vertical-align:middle}#createBtn___UiGiR .anticon+span{margin-left:0}#createBtn___UiGiR span+.anticon{margin-left:0}.col___yKN-b{flex:0 0 20%;max-width:20%}@media screen and (max-width:1200px){.col___yKN-b{max-width:18%}}