._ppcard___1sR0b{background-color:#fff}._ppcard___1sR0b .title___2fjZe{height:2.5rem;margin-bottom:0;padding-left:1.063rem;font-weight:400;font-size:1.25rem;line-height:2.5rem;border-left:.375rem solid #1727c2}.shadow___3SaOl{height:47.5rem;margin-top:1.75rem;background-color:#fff;box-shadow:0 .125rem .063rem 0 rgba(0,0,0,.26)}.main___IOVJu{background:#fff}.block_l___1ywNh{float:left;width:40%}.block_r___Sg4L7{float:left;width:60%}.goup___1vznA{float:left}.container___2RXc3{width:100%;padding:1.25rem 2.5rem 1.875rem;background-repeat:no-repeat;background-size:cover}.ant-pro-basicLayout-content{display:flex;margin:0}