(self.webpackChunkPaddleLabel_Frontend=self.webpackChunkPaddleLabel_Frontend||[]).push([[571],{91562:function(d,n,t){"use strict";t.r(n);var e=t(57106),s=t(99683),o=t(57663),a=t(71577),u=t(67294),E=t(48971),_=t(85893),P=function(){return(0,_.jsx)(s.ZP,{status:"404",title:"404",subTitle:"Sorry, the page you visited does not exist.",extra:(0,_.jsx)(a.Z,{type:"primary",onClick:function(){return E.m8.push("/")},children:"Back Home"})})};n.default=P}}]);
