.card___CFZWU{width:12.5rem;border:none;cursor:pointer}.card___CFZWU .ant-card-body{margin:.875rem 0 0;padding:.438rem;font-weight:400;font-size:1.1rem;text-align:center;border:.125rem solid #f0f0f0;border-radius:.125rem}.card___CFZWU .thumbnail___1gIPM{width:100%;height:15rem;box-shadow:0 .125rem .375rem 0 rgba(0,0,0,.27)}.card___CFZWU .button___g0lkO{width:12.5rem;height:2.5rem;margin-top:.875rem;padding:0;font-size:.85rem;border-radius:.125rem;box-shadow:0 .125rem .375rem 0 rgba(0,0,0,.27)}.ppcard___27hGd{padding:1.25rem 0;background-color:#fff;box-shadow:0 .125rem .063rem 0 rgba(0,0,0,.26)}.ppcard___27hGd .title___22R8f{height:2.5rem;margin-bottom:0;padding-left:1.063rem;font-weight:400;font-size:1.25rem;line-height:2.5rem;border-left:.375rem solid #1727c2}.container___2RXc3{width:100%;padding:1.25rem 2.5rem 1.875rem;background-repeat:no-repeat;background-size:cover}.ant-pro-basicLayout-content{display:flex;margin:0}.col___yKN-b{flex:0 0 20%;max-width:20%}@media screen and (max-width:1200px){.col___yKN-b{max-width:18%}}