.container___G0FNe{position:relative;display:flex;width:100%;padding:0;background-color:#fafafa}.ant-pro-basicLayout-content{display:flex;margin:0}.toolBarButtonContainerWrapper___1SHWL{cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.toolBarButtonContainerWrapper___1SHWL :hover{background-color:#eeeff1}.toolBarButtonContainerWrapper___1SHWL .toolBarButtonContainer___1jKUz{width:100%;min-height:4.063rem}.toolBarButtonContainerWrapper___1SHWL .toolBarButtonContainer___1jKUz .toolBarButton___3Ouq1{width:4.063rem;text-align:center}.toolBarButtonContainerWrapper___1SHWL .toolBarButtonContainer___1jKUz .toolBarButton___3Ouq1 .buttonText___10ls7{font-size:.75rem;text-align:center}.toolBarButtonContainerWrapperActive___3AA28{background-color:#e5e5e5}.toolBarButtonContainerWrapperDisabled___39n5R{color:#b3b3b3;cursor:not-allowed}.toolBarButtonContainerWrapperDisabled___39n5R :hover{background-color:transparent}.toolBarButtonImgDisabled___rRnhA{opacity:.3}.leftToolbar___1xi4t{width:70px;padding-top:10px;background-color:#fff;border-right:4px solid #ddd}.rightToolbar___3dNSB{width:70px;padding-top:10px;background-color:#fff;border-left:4px solid #ddd}.labelList___jvD05{border-right:0;border-bottom:.25rem solid #ddd;border-left:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.labelList___jvD05 .ant-list-footer,.labelList___jvD05 .ant-list-header{padding:0!important}.labelList___jvD05 .ant-list-item{padding:.5rem .813rem!important;font-size:.75rem}.labelList___jvD05 .listHeader___UM5UF{height:2.188rem;padding-left:1.25rem;font-size:.875rem;line-height:2.188rem;background-color:#e8eced}.labelList___jvD05 .eye___II3Po{display:block;width:1.5rem;height:1.5rem;background-repeat:no-repeat;background-size:1.5rem 1.5rem}.labelList___jvD05 .roundBall___j752J{width:1.125rem;height:1.125rem;margin-left:.375rem;background-size:100% 100%;border-radius:50%}.listItem___vtDXn .ant-space-item{position:relative}.listItem___vtDXn .eye___AsLBS{display:block;width:1.5rem;height:1.5rem;background-repeat:no-repeat;background-size:1.5rem 1.5rem}.listItem___vtDXn .roundBall___1J_R3{width:1.125rem;height:1.125rem;margin-left:.375rem;background-size:100% 100%;border-radius:50%}.listItem___vtDXn .popover___1x-2R{top:1.125rem!important;padding:0}.listItem___vtDXn .popover___1x-2R .ant-popover-inner{background:none;border:none;box-shadow:none}.listItem___vtDXn .popover___1x-2R .ant-popover-arrow{top:.25rem}.listItem___vtDXn .delete___WHPf2{display:block;width:1.1rem;height:1.1rem;background-image:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA8AAAAPCAYAAAA71pVKAAAAAXNSR0IArs4c6QAAAERlWElmTU0AKgAAAAgAAYdpAAQAAAABAAAAGgAAAAAAA6ABAAMAAAABAAEAAKACAAQAAAABAAAAD6ADAAQAAAABAAAADwAAAAAHNtsJAAAAw0lEQVQoFaWTwQ3CMBAEIyQogh+V4DJohL9boBX6gDJ40AEvmLF8UWLlEeKVVqc9X9a+szMMc+yQF3iHL/ipUW3e9UWcyD7gE1p4hIca1eZdt24GE+6U4R4uwXyG1o0GHkXHDNcgU2R9aSGOFDveWDjDKRLCvLAuWivD0SDgh28YBqnRyHGopQeHM0VCaHCtMYyQBdbbe7kOp9rCD7/Q2MJ6r7Fv53gA4f5Xz13T7rpnj7v5hUWvGmx622FgC85g1V/1A95nPRoOwStJAAAAAElFTkSuQmCC);background-repeat:no-repeat;background-position:50%;background-size:.9rem .9rem}.listItem___vtDXn .delete___WHPf2:hover{background-image:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA8AAAAPCAYAAAA71pVKAAAAAXNSR0IArs4c6QAAAERlWElmTU0AKgAAAAgAAYdpAAQAAAABAAAAGgAAAAAAA6ABAAMAAAABAAEAAKACAAQAAAABAAAAD6ADAAQAAAABAAAADwAAAAAHNtsJAAABbklEQVQoFZ2TPU7DQBCFn8PfAaCi4wQ0QIMEOUMUXOQA3CBpaGhonIpLuAiIMwASDSRIFNR0KRA+QEDEfM/2ok0BEow02tk37+2OZ8eJIiul1lhKgXr4Dr6OFziwcoBRIs2JKyOu7Unaepcu2RnLlqTbNeltJm18SgdgA7xclbrb0gtxbRY+SFP8lCtWAh6vxp03z/wq51IBJk7E5J/i5oCJdeEbE2o9s+BeOscPYzGCtnFjDS+hkrTF3s3JaMaHk9gVhItwgIVgI+O4u2hehvdEcvoobToRzAL8Fe97DQeFvPngU4tnzxJNXDQLEZVeFzOS+da57MLPERNItNn3Kd/P40MWetDwC4vHzTsSVg0zcUQ3j3alIXEa98Cchj+2OMcH0ft2LNyTbkzkgGsWT10H96h5DlxRLoj/fudlSpozMV1G845vrd4xejaQ2nwjF52wO6a7+9bhtXnk/jrb32If4U/w5BB6cCjg97/qC+BvhVGySKhxAAAAAElFTkSuQmCC)}.listItemActive___3FRb7{background-color:#a9aee1}.roundBall___1pqu1{width:1.125rem;height:1.125rem;margin-left:.375rem;background-size:100% 100%;border-radius:50%}.popover___1-PR6{top:1.125rem!important;padding:0}.popover___1-PR6 .ant-popover-inner{background:none;border:none;box-shadow:none}.popover___1-PR6 .ant-popover-arrow{top:.25rem}