.container___1sYa-{display:flex;flex-direction:column;height:100vh;overflow:auto;background:#f0f2f5}.container___1sYa- .ant-pro-form-login-logo{width:6.688rem}.lang___l6cji{width:100%;height:2.5rem;line-height:2.75rem;text-align:right}.lang___l6cji .ant-dropdown-trigger{margin-right:1.5rem}.content___2zk1-{flex:1 1;padding:2rem 0}@media (min-width:768px){.container___1sYa-{background-image:url(https://gw.alipayobjects.com/zos/rmsportal/TVYTbAXWheQpRcWDaDMu.svg);background-repeat:no-repeat;background-position:center 6.875rem;background-size:100%}.content___2zk1-{padding:2rem 0 1.5rem}}.icon___rzGKO{margin-left:.5rem;color:rgba(0,0,0,.2);font-size:1.5rem;vertical-align:middle;cursor:pointer;transition:color .3s}.icon___rzGKO:hover{color:#1727c2}#logo___2hXsy{width:6.688rem}