(self.webpackChunkPaddleLabel_Frontend=self.webpackChunkPaddleLabel_Frontend||[]).push([[540],{41435:function(w,l,r){"use strict";r.d(l,{Z:function(){return c}});var d=r(94663),a=r(80112);function v(o){return Function.toString.call(o).indexOf("[native code]")!==-1}var y=r(18597);function u(o,n,s){return(0,y.Z)()?u=Reflect.construct:u=function(e,b,f){var i=[null];i.push.apply(i,b);var h=Function.bind.apply(e,i),p=new h;return f&&(0,a.Z)(p,f.prototype),p},u.apply(null,arguments)}function c(o){var n=typeof Map=="function"?new Map:void 0;return c=function(t){if(t===null||!v(t))return t;if(typeof t!="function")throw new TypeError("Super expression must either be null or a function");if(typeof n!="undefined"){if(n.has(t))return n.get(t);n.set(t,e)}function e(){return u(t,arguments,(0,d.Z)(this).constructor)}return e.prototype=Object.create(t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),(0,a.Z)(e,t)},c(o)}}}]);
